import os
import time

from ConfigParser import SafeConfigParser

from shub.click_utils import fail
from shub.utils import pwd_hg_version, pwd_git_version


def get_project(target, project):
    project = project or target.get('project')
    if not project:
        raise fail("Error: Missing project id")
    return str(project)


def get_targets():
    cfg = get_config()
    baset = dict(cfg.items('deploy')) if cfg.has_section('deploy') else {}
    baset.setdefault('url', 'https://dash.scrapinghub.com/api/scrapyd/')
    targets = {}
    targets['default'] = baset
    for x in cfg.sections():
        if x.startswith('deploy:'):
            t = baset.copy()
            t.update(cfg.items(x))
            targets[x[7:]] = t
    return targets


def get_target(name):
    try:
        return get_targets()[name]
    except KeyError:
        raise fail("Unknown target: %s" % name)


def get_version(target, version):
    version = version or target.get('version')
    if version == 'HG':
        return pwd_hg_version()
    elif version == 'GIT':
        return pwd_git_version()
    elif version:
        return str(version)
    else:
        return str(int(time.time()))


def get_config(use_closest=True):
    """Get Scrapy config file as a SafeConfigParser"""
    sources = get_sources(use_closest)
    cfg = SafeConfigParser()
    cfg.read(sources)
    return cfg


def get_sources(use_closest=True):
    xdg_config_home = os.environ.get('XDG_CONFIG_HOME') or \
        os.path.expanduser('~/.config')
    sources = ['/etc/scrapy.cfg', r'c:\scrapy\scrapy.cfg',
               xdg_config_home + '/scrapy.cfg',
               os.path.expanduser('~/.scrapy.cfg')]
    if use_closest:
        sources.append(closest_scrapy_cfg())
    return sources


def closest_scrapy_cfg(path='.', prevpath=None):
    """Return the path to the closest scrapy.cfg file by traversing the current
    directory and its parents
    """
    if path == prevpath:
        return ''
    path = os.path.abspath(path)
    cfgfile = os.path.join(path, 'scrapy.cfg')
    if os.path.exists(cfgfile):
        return cfgfile
    return closest_scrapy_cfg(os.path.dirname(path), path)



import click
from click import ClickException
from hubstorage import HubstorageClient
from shub.utils import find_api_key, is_valid_jobid


@click.command(help='Get items of a given job on Scrapy Cloud')
@click.pass_context
@click.argument('job_id')
def cli(context, job_id):
    if not is_valid_jobid(job_id):
        raise ClickException('Invalid job ID. Job ID must be: projectid/spiderid/jobid')
    apikey = find_api_key()
    if not apikey:
        raise ClickException('Scrapinghub API key not found: please, run \'scrapy login\' first')
    for item in fetch_items_for_job(apikey, job_id):
        click.echo(item)


def fetch_items_for_job(apikey, job_id):
    hc = HubstorageClient(auth=apikey)
    job = hc.get_job(job_id)
    if not job.metadata:
        raise ClickException('The job {} doesn\'t exist'.format(job_id))
    return job.items.iter_values()



import click
import os
import requests
from shub import auth
from six.moves import input
from shub.scrapycfg import get_config


@click.command(help='add Scrapinghug API key into the netrc file')
@click.pass_context
def cli(context):
    if auth.get_key_netrc():
        click.echo("You're already logged in. To change credentials, use 'shub logout' first.")
        return 0

    cfg_key = _find_cfg_key()
    key = _get_apikey(suggestion=cfg_key)
    auth.write_key_netrc(key)


def _get_apikey(suggestion=''):
    suggestion_txt = ' (%s)' % suggestion if suggestion else ''
    click.echo('Enter your API key from https://dash.scrapinghub.com/account/apikey')
    key = ''
    while True:
        key = input('API key%s: ' % suggestion_txt) or suggestion
        click.echo("Validating API key...")
        if _is_valid_apikey(key):
            click.echo("API key is OK, you are logged in now.")
            return key
        else:
            click.echo("API key failed, try again.")


def _is_valid_apikey(key):
    validate_api_key_endpoint = "https://dash.scrapinghub.com/api/v2/users/me"
    r = requests.get("%s?apikey=%s" % (validate_api_key_endpoint, key))
    return r.status_code == 200


def _find_cfg_key():
    cfg_key = _read_scrapy_cfg_key()
    if cfg_key:
        return cfg_key

    envkey = os.getenv("SHUB_APIKEY")
    if envkey:
        return envkey


def _read_scrapy_cfg_key():
    try:
        cfg = get_config()

        if cfg.has_section('deploy'):
            deploy = dict(cfg.items('deploy'))
            key = deploy.get('username')

            if key:
                return key
    except:
        return



import click
from click import ClickException
from scrapinghub import Connection, APIError
from shub import scrapycfg
from shub.utils import find_api_key


@click.command(help='Schedule a spider to run on Scrapy Cloud')
@click.pass_context
@click.argument('spider', type=click.STRING)
@click.option('-p', '--project-id', help='the project ID', type=click.INT)
@click.option('-a', '--argument', help='argument for the spider (-a name=value)', multiple=True)
def cli(context, project_id, spider, argument):
    apikey = find_api_key()
    if not apikey:
        raise ClickException('Scrapinghub API key not found: please, run \'scrapy login\' first')
    project_id = project_id or get_project_id_from_config()
    job_key = schedule_spider(apikey, project_id, spider, argument)
    click.echo(
        'Spider {} scheduled, watch it running here:\n'
        'https://dash.scrapinghub.com/p/{}/job/{}/{}'.format(spider, *job_key.split('/'))
    )


def schedule_spider(apikey, project_id, spider, arguments=()):
    conn = Connection(apikey)
    try:
        return conn[project_id].schedule(spider, **dict(x.split('=') for x in arguments))
    except APIError as e:
        raise ClickException(e.message)


def get_project_id_from_config():
    from six.moves.configparser import NoSectionError, NoOptionError
    try:
        conf = scrapycfg.get_config()
        return conf.get('deploy', 'project')
    except (NoSectionError, NoOptionError):
        raise ClickException('missing project id')
